@@ -5,25 +5,19 @@ import os
 from typing import Optional
 from urllib.parse import urlparse
 
-from uplink.auth import BearerToken
-
 from bpkio_api.caching import init_cache
 from bpkio_api.consumer import BpkioSdkConsumer
 from bpkio_api.credential_provider import TenantProfile, TenantProfileProvider
 from bpkio_api.defaults import DEFAULT_FQDN
-from bpkio_api.endpoints import (
-    CategoriesApi,
-    ConsumptionApi,
-    ServicesApi,
-    SourcesApi,
-    TenantsApi,
-    TranscodingProfilesApi,
-    UsersApi,
-)
-from bpkio_api.exceptions import InvalidApiKeyFormat
+from bpkio_api.endpoints import (CategoriesApi, ConsumptionApi, ServicesApi,
+                                 SourcesApi, TenantsApi,
+                                 TranscodingProfilesApi, UsersApi)
+from bpkio_api.exceptions import (BroadpeakIoApiError, InvalidApiKeyFormat,
+                                  InvalidEndpointError)
 from bpkio_api.helpers.recorder import SessionRecorder
 from bpkio_api.mappings import model_to_endpoint
 from bpkio_api.models import Tenant
+from uplink.auth import BearerToken
 
 
 class BroadpeakIoApi(BpkioSdkConsumer):
@@ -37,6 +31,7 @@ class BroadpeakIoApi(BpkioSdkConsumer):
         session_file: Optional[str] = None,
         **kwargs,
     ):
+        self.use_cache = use_cache
         if tenant and api_key:
             raise ValueError("You can't specify both tenant and api_key")
 
@@ -101,6 +96,17 @@ class BroadpeakIoApi(BpkioSdkConsumer):
             base_url, auth=BearerToken(self._api_key), **kwargs
         )
 
+        # Optional endpoints
+        try:
+            from bpkio_api_admin.addons import add_admin_endpoints
+
+            add_admin_endpoints(
+                self, base_url, auth=BearerToken(self._api_key), **kwargs
+            )
+        except Exception as e:
+            # print(e)
+            pass
+
     @staticmethod
     def _parse_api_key(candidate: str) -> dict:
         """Parses an API Key (token) and extract the information it contains.
@@ -149,7 +155,7 @@ class BroadpeakIoApi(BpkioSdkConsumer):
                 name="Tenant 1",
                 description="Tenant 1",
                 commercialPlan="ADMIN",
-                state="Enabled",
+                state="enabled",
                 creationDate=datetime.datetime.min,
                 updateDate=datetime.datetime.min,
             )
@@ -181,16 +187,46 @@ class BroadpeakIoApi(BpkioSdkConsumer):
         return fqdn
 
     @staticmethod
-    def is_correct_entrypoint(url: str, api_key: str) -> bool | str:
+    def is_correct_entrypoint(url: str, api_key: str, verify_ssl) -> bool | str:
         """Checks that the URL is a valid Broadpeak.io entrypoint"""
 
         try:
-            api = BroadpeakIoApi(api_key=api_key, fqdn=url)
+            api = BroadpeakIoApi(api_key=api_key, fqdn=url, verify_ssl=verify_ssl)
             api.get_self_tenant()
             return True
         except Exception:
             return False
 
+    def test_access(self) -> bool:
+        """Tests if the API key allows to access the API"""
+        tenant_id = self.get_tenant_id()
+        try:
+            if tenant_id == 1:
+                self.users.list()
+            else:
+                self.tenants.retrieve_self()
+        except BroadpeakIoApiError as e:
+            raise BroadpeakIoApiError(
+                500,
+                "Unable to access the API. Check that your API key is valid",
+                None,
+                None,
+            )
+
+    # -- API traversal methods
+
     def root_endpoint_for_resource(self, resource: object) -> object:
         """Returns the root endpoint for a given resource"""
         return model_to_endpoint(api=self, model=type(resource))
+
+    def get_endpoint_from_path(self, path: list):
+        try:
+            endpoint = self
+            for p in path:
+                endpoint = getattr(endpoint, p)
+            return endpoint
+        except AttributeError as e:
+            raise InvalidEndpointError(
+                "The endpoint you are trying to access does not exist or is restricted",
+                original_message=e.args[0],
+            )
