@@ -1,26 +1,16 @@
 from typing import Any, List, Tuple
 from urllib.parse import urlparse, urlunparse
 
-from uplink import (
-    Body,
-    Query,
-    delete,
-    get,
-    json,
-    post,
-    put,
-    response_handler,
-    returns,
-)
-
-from bpkio_api.consumer import BpkioSdkConsumer
 from bpkio_api.caching import cache_api_results
+from bpkio_api.consumer import BpkioSdkConsumer
 from bpkio_api.exceptions import BroadpeakIoHelperError, ResourceExistsError
 from bpkio_api.helpers.list import collect_from_ids, get_all_with_pagination
 from bpkio_api.helpers.objects import find_duplicates_of
 from bpkio_api.helpers.search import SearchMethod, search_array_with_filters
 from bpkio_api.models import Sources as S
 from bpkio_api.response_handler import postprocess_response
+from uplink import (Body, Query, delete, get, json, post, put,
+                    response_handler, returns)
 
 from .enums import UpsertOperationType
 
@@ -39,6 +29,7 @@ class SourcesApi(BpkioSdkConsumer):
             parent_api=self, base_url=base_url, **kwargs
         )
         self.slate = SlateSourcesApi(parent_api=self, base_url=base_url, **kwargs)
+        self.origin = OriginSourcesApi(parent_api=self, base_url=base_url, **kwargs)
 
     def _mappings(self, model):
         match model:
@@ -150,15 +141,15 @@ class SourcesApi(BpkioSdkConsumer):
                     return self.slate.retrieve(source_id)
                 case S.SourceType.AD_SERVER:
                     return self.ad_server.retrieve(source_id)
+                case S.SourceType.ORIGIN:
+                    return self.origin.retrieve(source_id)
                 case _:
                     raise BroadpeakIoHelperError(
-                        status_code=404,
                         message=f"Source type not supported: {source.type}",
                         original_message="",
                     )
         except IndexError as e:
             raise BroadpeakIoHelperError(
-                status_code=404,
                 message=f"There is no source with ID {source_id}",
                 original_message=e.args[0],
             )
@@ -181,7 +172,6 @@ class SourcesApi(BpkioSdkConsumer):
         source = self.retrieve(source_id)
         if not source:
             raise BroadpeakIoHelperError(
-                status_code=404,
                 message=f"There is no source with ID {source_id}",
             )
         endpoint = self._mappings(source)
@@ -299,7 +289,7 @@ class AssetSourcesApi(BpkioSdkConsumer):
 
     def upsert(
         self, source: S.AssetSourceIn, if_exists: str = "retrieve"
-    ) -> Tuple[S.AssetSource, int]:
+    ) -> Tuple[S.AssetSource, UpsertOperationType]:
         """Conditionally create, retrieve or update an Asset source"""
         return self.parent_api.upsert(
             source, unique_fields=["url"], if_exists=if_exists
@@ -362,7 +352,7 @@ class LiveSourcesApi(BpkioSdkConsumer):
 
     def upsert(
         self, source: S.LiveSourceIn, if_exists: str | None = None
-    ) -> Tuple[S.LiveSource, int]:
+    ) -> Tuple[S.LiveSource, UpsertOperationType]:
         """Conditionally create, retrieve or update a Live source"""
         return self.parent_api.upsert(
             source, unique_fields=["url"], if_exists=if_exists
@@ -425,7 +415,7 @@ class AssetCatalogSourcesApi(BpkioSdkConsumer):
 
     def upsert(
         self, source: S.AssetCatalogSourceIn, if_exists: str | None = None
-    ) -> Tuple[S.AssetCatalogSource, int]:
+    ) -> Tuple[S.AssetCatalogSource, UpsertOperationType]:
         """Conditionally create, retrieve or update an Asset Catalog source"""
         return self.parent_api.upsert(
             source, unique_fields=["url"], if_exists=if_exists
@@ -488,11 +478,9 @@ class AdServerSourcesApi(BpkioSdkConsumer):
 
     def upsert(
         self, source: S.AdServerSourceIn, if_exists: str | None = None
-    ) -> Tuple[S.AdServerSource, int]:
+    ) -> Tuple[S.AdServerSource, UpsertOperationType]:
         """Conditionally create, retrieve or update an Ad Server source"""
-        return self.parent_api.upsert(
-            source, unique_fields=["url", "queries"], if_exists=if_exists
-        )
+        return self.parent_api.upsert(source, unique_fields=[], if_exists=if_exists)
 
     def create_from_url(self, name: str, url: str) -> S.AdServerSource:
         """Convenience function to create an ad-server from a full URL
@@ -517,8 +505,20 @@ class AdServerSourcesApi(BpkioSdkConsumer):
             return first_part, second_part
 
         parts = split_url(url)
+        queries = parts[1] or ""
+        individual_queries = queries.split("&") if queries else []
+
+        query_params = [
+            S.AdServerQueryParameter(
+                type=S.AdServerQueryParameterType.custom, name=qp_name, value=qp_value
+            )
+            for qp_name, qp_value in (
+                indiv_qp.split("=") for indiv_qp in individual_queries
+            )
+        ]
+
         ad_server_source = S.AdServerSourceIn(
-            name=name, url=parts[0], queries=parts[1], description=None
+            name=name, url=parts[0], queryParameters=query_params, description=None
         )
         return self.create(ad_server_source)
 
@@ -579,7 +579,7 @@ class SlateSourcesApi(BpkioSdkConsumer):
 
     def upsert(
         self, source: S.SlateSourceIn, if_exists: str | None = None
-    ) -> Tuple[S.SlateSource, int]:
+    ) -> Tuple[S.SlateSource, UpsertOperationType]:
         """Conditionally create, retrieve or update a slate source"""
         return self.parent_api.upsert(
             source, unique_fields=["url"], if_exists=if_exists
@@ -608,3 +608,66 @@ class SlateSourcesApi(BpkioSdkConsumer):
 
         sources = self.list()
         return search_array_with_filters(sources, filters=filters)
+
+
+# === ORIGIN SOURCES ===
+
+
+@response_handler(postprocess_response)
+class OriginSourcesApi(BpkioSdkConsumer):
+    def __init__(self, parent_api: SourcesApi, base_url="", **kwargs):
+        super().__init__(base_url, **kwargs)
+        self.parent_api = parent_api
+
+    @returns.json(S.OriginSource)
+    @get("sources/origin/{source_id}")
+    def retrieve(self, source_id):
+        """Get a single origin source, by ID"""
+
+    @json
+    @returns.json(S.OriginSource)
+    @post("sources/origin")
+    def create(self, source: Body(type=S.OriginSourceIn)) -> S.OriginSource:  # type: ignore
+        """Create a new origin source"""
+
+    @json
+    @returns.json(S.OriginSource)
+    @put("sources/origin/{source_id}")
+    def update(self, source_id: int, source: Body(type=S.OriginSourceIn)) -> S.OriginSource:  # type: ignore
+        """Update an origin source"""
+
+    @delete("sources/origin/{source_id}")
+    def delete(self, source_id: int):
+        """Delete a origin source, by ID"""
+
+    def upsert(
+        self, source: S.OriginSourceIn, if_exists: str | None = None
+    ) -> Tuple[S.OriginSource, UpsertOperationType]:
+        """Conditionally create, retrieve or update an origin source"""
+        return self.parent_api.upsert(
+            source, unique_fields=["url"], if_exists=if_exists
+        )
+
+    @cache_api_results("list_origins")
+    def list(self) -> List[S.OriginSource]:
+        """List all Origin sources"""
+        sparse_sources = self.parent_api.search(
+            S.SourceType.ORIGIN, field="type", method=SearchMethod.STRICT
+        )
+        return collect_from_ids(
+            ids=[src.id for src in sparse_sources], get_fn=self.retrieve
+        )
+
+    def search(
+        self,
+        value: Any | None = None,
+        field: str | None = None,
+        method: SearchMethod = SearchMethod.STRING_SUB,
+        filters: List[Tuple[Any, str | None, SearchMethod | None]] | None = None,
+    ) -> List[S.OriginSource]:
+        """Searches the list of Origin sources for those matching a particular filter query"""
+        if not filters:
+            filters = [(value, field, method)]
+
+        sources = self.list()
+        return search_array_with_filters(sources, filters=filters)
