@@ -12,23 +12,39 @@ class CodecStringParser:
             string = string[1:-1]
 
         codecstrings = [c.strip() for c in string.split(",")]
-        return [CodecStringParser.parse_codec_string(c) for c in codecstrings]
+        out = []
+        for c in codecstrings:
+            cinfo = CodecStringParser.parse_codec_string(c)
+            out.append(cinfo)
+
+        return out
 
     @staticmethod
     def parse_codec_string(codecstring: str) -> Dict:
         (codec, *codec_data) = codecstring.split(".")
+        codec_info = None
 
         match codec:
             case "avc1" | "avc3":
-                return CodecStringParser._parse_avc(codec_data[0])
+                # Treatment of legacy codec strings
+                if len(codec_data) == 2:
+                    codec_info = CodecStringParser._parse_avc(
+                        f"{codec_data[0]}.{codec_data[1]}", legacy=True
+                    )
+                else:
+                    codec_info = CodecStringParser._parse_avc(codec_data[0])
             case "mp4a":
-                return CodecStringParser._parse_aac(codec_data)
+                codec_info = CodecStringParser._parse_aac(codec_data)
             case _:
                 raise Exception(f"Codec '{codec}' invalid or not yet implemented")
 
+        codec_info["codecstring"] = codecstring
+        return codec_info
+
     @staticmethod
-    def _parse_avc(codec_data):
-        H264Profile = {
+    def _parse_avc(codec_data, legacy=False):
+        H264ProfileLegacy = {"66": "Baseline", "77": "Main", "100": "High"}
+        H264ProfileRFC6381 = {
             "42": "Baseline",
             "4D": "Main",
             "58": "Extended",
@@ -36,9 +52,9 @@ class CodecStringParser:
             "6E": "High 10",
             "7A": "High 4:2:2",
             "F4": "High 4:4:4",
-            "2C": "CAVLC 4:4:4"
+            "2C": "CAVLC 4:4:4",
         }
-        H264Level = {
+        H264LevelRFC6381 = {
             "0A": "1",
             "0B": "1.1",
             "0C": "1.2",
@@ -61,13 +77,21 @@ class CodecStringParser:
         }
 
         try:
-            profile, level = codec_data[:2].upper(), codec_data[4:6].upper()
+            if legacy:
+                profile, level = codec_data.split(".")
+                pro = H264ProfileLegacy[profile]
+                lev = f"{level[0]}.{level[1]}"
+            else:
+                profile, level = codec_data[:2].upper(), codec_data[4:6].upper()
+                pro = H264ProfileRFC6381[profile]
+                lev = H264LevelRFC6381[level]
+
             return dict(
                 cc="H264",
                 type="video",
                 codec="AVC/H.264",
-                profile=H264Profile[profile],
-                level=H264Level[level],
+                profile=pro,
+                level=lev,
             )
         except Exception:
             raise Exception(f"Invalid or unsupported H264 codec data: {codec_data}")
@@ -84,11 +108,15 @@ class CodecStringParser:
             "69": "MPEG-2 Audio Part 3",
             "6B": "MPEG-1 Part 3",
         }
-        AAC_MODE = {"1": "Main", "2": "Low-Complexity AAC"}
+        AAC_MODE = {
+            "1": "Main",
+            "2": "AAC-LC (Low-Complexity)",
+            "5": "SBR (Spectral Band Replication)",
+        }
 
         try:
             return dict(
                 cc="AAC", type="audio", codec=AAC_OTI[oti.upper()], mode=AAC_MODE[mode]
             )
         except Exception:
-            raise Exception(f"Invalid or unsupported H264 codec data: {codec_data}")
+            raise Exception(f"Invalid or unsupported AAC codec data: {codec_data}")
