@@ -1,13 +1,15 @@
+from bpkio_api.models.MediaFormat import MediaFormat
+
 from .generic import ContentHandler
 
 
 class MP4Handler(ContentHandler):
+    media_format = MediaFormat.MP4
     content_types = ["video/mp4"]
-    file_extensions = [".mp4", ".mov"]
 
     def __init__(self, url, content, **kwargs):
         self.url = url
-        self.content = content
+        self._content = content
 
     def read(self):
         return "Handling MP4 file."
