@@ -1,11 +1,7 @@
-import logging
-
 import bpkio_api.helpers.handlers as handlers
 from bpkio_api.exceptions import BroadpeakIoHelperError
 from bpkio_api.models.Sources import SourceType
 
-logger = logging.getLogger(__name__)
-
 
 class SourceTypeDetector:
     @staticmethod
@@ -27,7 +23,6 @@ class SourceTypeDetector:
 
             case _:
                 raise BroadpeakIoHelperError(
-                    status_code=404,
                     message=f"Unsupported document type for source at {url}",
                     original_message=None,
                 )
